#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
"""
    GESTIKK
    Version 0.4

    Gestikk, a mouse gesture recognition application
    Copyright (C) 2007 Friedrich 'fred.reichbier' Weber (2007-10-08)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""

from __future__ import division
import os.path
import wx
import signal
import time
import subprocess
import sys
from thread import start_new_thread
import ConfigParser
from gestikk_tools import *

_ = init_gettext().gettext

wx.SetDefaultPyEncoding("iso-8859-1")

class GestikkTaskBarIcon(wx.TaskBarIcon):
    """ showing a task bar icon """
    def __init__(self, close_func):
        wx.TaskBarIcon.__init__(self)
        #self.id_close = 2

        self.Bind(wx.EVT_MENU, self.evt_close_gestikk, id=wx.ID_CLOSE)
        self.Bind(wx.EVT_MENU, self.evt_about, id=wx.ID_ABOUT)
        self.Bind(wx.EVT_MENU, self.evt_configuration, id=wx.ID_PROPERTIES)
        self.close_func = close_func

    def get_bitmap(self, fname):
        return wx.Image(fname, wx.BITMAP_TYPE_PNG).ConvertToBitmap()

    def evt_about(self, evt):
        """ show about dialog """
        wx.AboutBox(about_dialog_info())

    def evt_configuration(self, evt):
        """ launch the configuration """
        subprocess.Popen([os.path.join(BASEDIR, 'gestikk.py'), '-c', '-q'])

    def CreatePopupMenu(self):
        menu = wx.Menu()
        # # Configuration
        item = wx.MenuItem(menu, wx.ID_PROPERTIES, _('Configuration'))
        menu.AppendItem(item)
        # # About
        item = wx.MenuItem(menu, wx.ID_ABOUT , _('About Gestikk...'))
        menu.AppendItem(item)
        # #
        menu.AppendSeparator()
        # # Close
        item = wx.MenuItem(menu, wx.ID_CLOSE, _('Close Gestikk'))
        menu.AppendItem(item)
        return menu

    def evt_close_gestikk(self, evt):
        evt.Skip()
        wx.CallAfter(self.close_func)

class MGFrame(wx.Frame):
    def __init__(self, logic, parent = None, id = -1, title = "Gestikk"):
        wx.Frame.__init__(self, parent, id, title)
        self.running = True
        start_new_thread(self.mouse_gesture_thread, ())
        self.gesture_positions = [] # (x,y), (x,y), ...
        self.logic = logic
        # #
        self.Bind(wx.EVT_CLOSE, self.evt_close)

    def check_mouse_position(self):
        """ Check position of mouse pointer """
        pos = wx.GetMousePosition()
        mstate = {'rightdown':wx.GetMouseState().RightDown(),
                  'leftdown':wx.GetMouseState().LeftDown(),
                  'middledown':wx.GetMouseState().MiddleDown(),
                  'ctrldown':wx.GetMouseState().ControlDown(),
                  'altdown':wx.GetMouseState().AltDown(),
                  'shiftdown':wx.GetMouseState().ShiftDown(),
                  'cmddown':wx.GetMouseState().CmdDown(),
                  'metadown':wx.GetMouseState().MetaDown()}.get(self.logic.current_switcher)
        if mstate: # Gesture done
            self.gesture_positions.append(pos)
        else: # button no longer pressed
            if self.gesture_positions: # end gesture
                # calculate path
                gesture = []
                tolerance = int(self.logic.config_get_option('Preferences', 'tolerance', '25'))
                for gid in range(1, len(self.gesture_positions)): # Let first out
                    gesture.append(calc_direction(self.gesture_positions[gid-1], \
                                                self.gesture_positions[gid], tolerance))
                # Remove zeros
                while (0,0) in gesture:
                    gesture.remove((0,0))
                # Delete duplicates
                gesture = remove_duplicates(gesture)
                self.gesture_positions = []
                # Is an action assigned to the gesture?
                gesture_repr = repr(gesture).replace(' ', '')[1:-1] # withour spaces and brackets
                if DEBUG:
                    print 'Gesture: %s' % gesture_repr
                if self.logic.gestures.has_key(gesture_repr):
                    if DEBUG:
                        print 'Starting action with gesture '+gesture_repr
                    self.logic.gestures[gesture_repr].do() # very easy :-)


    def mouse_gesture_thread(self):
        """ Thread for gesture recognition """
        while self.running:
            wx.CallAfter(wx.YieldIfNeeded)
            wx.CallAfter(self.check_mouse_position)
            time.sleep(0.25)

    def evt_close(self, evt):
        self.tbIcon.Destroy()
        self.running = False
        time.sleep(0.26)
        evt.Skip(True)

    def sig_reload_gestures(self, signum, frame):
        """ we should reload the gestures file - we received a SIGUSR1 signal,
            probably from the gestikk configuration frame. """
        self.logic.reload_gestures()
        self.logic.reload_config()

def main(debug):
    """
        :param debug: Debug mode on?
    """
    global DEBUG
    DEBUG = debug
    app = wx.PySimpleApp()
    logic = Logic()
    frame = MGFrame(logic)
    # # # Taskbar Icon # # #
    signal.signal(signal.SIGUSR1, frame.sig_reload_gestures)
    if os.path.exists(os.path.join('/usr/share/icons', 'gestikk.xpm')):
        icon = wx.Icon(os.path.join('/usr/share/icons', 'gestikk.xpm'), wx.BITMAP_TYPE_XPM)
    else:
        icon = wx.Icon(os.path.join(BASEDIR, 'icons', 'gestikk.xpm'), wx.BITMAP_TYPE_XPM)
    tbi = GestikkTaskBarIcon(frame.Close)
    tbi.SetIcon(icon, 'Gestikk')
    if not tbi.IsIconInstalled():
        print _('Taskbar Icon Error!')
    frame.tbIcon = tbi
    # # # # # # # # # # # #
    app.MainLoop()

if __name__ == '__main__':
    main(False)
